class MqApis:
    MQ_QUEUE_LIST = "mq.queue.list"
    MQ_QUEUE_VIEW_MESSAGE = "mq.queue.view.message"
    MQ_QUEUE_VIEW_SESSION = "mq.queue.view.session"
    MQ_QUEUE_FORCE_CLEAR = "mq.queue.force.clear"
    MQ_QUEUE_FORCE_DELETE = "mq.queue.force.delete"
    MQ_QUEUE_FORCE_DISTRIBUTE = "mq.queue.force.distribute"